SIGACTS <- FALSE
DF <- FALSE
proj.home <- "/home/usr0/z41440a/afghanistan/"

if(SIGACTS) {
    library("maptools")
    if(DF) {
        ifname <- paste(proj.home, "pts_df.shp", sep="")
    } else ifname <- paste(proj.home, "pts.shp", sep="")

    tmp <- readShapeSpatial(ifname)
    final <- tmp@data
    dates.tmp <- as.Date(final[,"date"], "%m/%d/%y")
    ids.tmp <- final[,"DISTID"]
    dates <- dates.tmp[dates.tmp >= as.Date("2004-01-01")] 
    ids <- as.character(ids.tmp[dates.tmp >= as.Date("2004-01-01")] )
} else {
    load(paste(proj.home, "final_dataset.RData", sep=""))
    dates <- as.Date(final$truedate, origin=as.Date("1960-01-01"))
    ids <- as.character(final$truedistrict)
}

nc <- as.integer(max(dates) - as.Date("2004-01-01")) + 1
nr <- length(unique(ids))

d.mat <- matrix(0, nrow=nr, ncol=nc, dimnames=list(sort(unique(ids)), as.character(as.Date("2004-01-01") + 0:(nc-1))))
d.mat.tal <- d.mat
d.mat.notal <- d.mat
d.mat.noprov <- d.mat
d.mat.tal.noprov <- d.mat

if(SIGACTS) {
    for(i in 1:length(dates)) {
        t <- dates[i]
        d.mat[ids[i], as.character(t)] <- d.mat[ids[i], as.character(t)] + 1
    }
} else for(i in 1:nrow(final)) {
    icn.wt <- 1/sum(final$incident_id == final[i,"incident_id"] & final[,"lat"] == final[i,"lat"])  # fractional weights generated here appear to be correct
    t <- dates[i]
    if (t >= as.Date("2004-01-01")) {
        d.mat[ids[i],as.character(t)] <- d.mat[ids[i],as.character(t)] + icn.wt
        if(is.na(final[i,"category"]) || final[i,"category"] != "provincial centroid")
            d.mat.noprov[ids[i],as.character(t)] <- d.mat.noprov[ids[i],as.character(t)] + icn.wt
        if((!is.na(final[i,"group_id"] == 22) & final[i,"group_id"] == 22) &
           (!is.na(final[i,"claim_flag"]) & final[i,"claim_flag"] == 1)) {
            d.mat.tal[ids[i],as.character(t)] <- d.mat.tal[ids[i],as.character(t)] + icn.wt
            if(is.na(final[i,"category"]) || final[i,"category"] != "provincial centroid")
                d.mat.tal.noprov[ids[i],as.character(t)] <- d.mat.tal.noprov[ids[i],as.character(t)] + icn.wt
        } else d.mat.notal[ids[i],as.character(t)] <- d.mat.notal[ids[i],as.character(t)] + icn.wt
    }
}

require("methods")
library("maptools")

sname <- paste(proj.home, "AFG_district_398/district398", sep="")
shp <- readShapePoly(sname)

stopifnot(!duplicated(shp@data$DISTID))
row.names(shp) <- as.character(shp@data[,"DISTID"])

#### graph
library("spdep")
tmp <- poly2nb(shp, queen=FALSE)
adj.mat <- nb2mat(tmp, style="B", zero.policy=TRUE)
colnames(adj.mat) <- row.names(adj.mat)

library("igraph") #ONE-BASED INDEXING
b.graph <- graph.adjacency(adj.mat, mode="undirected", diag=FALSE)
xy.tmp <- coordinates(shp)
V(b.graph)$lon <- xy.tmp[, 1]
V(b.graph)$lat <- xy.tmp[, 2]

stopifnot(is.connected(b.graph))

V(b.graph)$label <- substring(V(b.graph)$name, first=nchar(V(b.graph)$name) - 2)

a.graph <- b.graph
#####

library("rgeos")
cents <- gCentroid(shp, byid=TRUE)
rownames(cents@coords) <- shp@data[,"DISTID"]


if(SIGACTS) {
    if(DF) {
        fname <- paste(proj.home, "af_new_SIGACTS_DF.RData", sep="")
    } else fname <- paste(proj.home, "af_new_SIGACTS.RData", sep="")
    save(final, a.graph, d.mat, file=fname)
} else {
    fname <- paste(proj.home, "af_newer_WITS.RData", sep="")
    save(final, a.graph, d.mat, d.mat.tal, d.mat.notal, d.mat.noprov, d.mat.tal.noprov, cents, file=fname)
}


####
table(final[,"group_name"])

