proj.home <- "/home/usr0/z41440a/afghanistan_ecma_final/"

USE16 <- TRUE  # should we use the 2016 version or the 2017 version?

#### works on mac, fails on linux?
library("readxl")
if(USE16) {
    fname <- paste(proj.home, "globalterrorismdb_0616dist.RData", sep="")
} else fname <- paste(proj.home, "globalterrorismdb_0617dist.RData", sep="")
load(fname)

CLAIMED.ONLY <- FALSE
TALIBAN.ONLY <- FALSE
stopifnot(USE16 + CLAIMED.ONLY + TALIBAN.ONLY <= 1)

COUNTRY <- "Afghanistan"
gtdb.use <- gtdb[(gtdb[,"country_txt"] == COUNTRY) & (gtdb[,"imonth"] != 0) & (gtdb[,"iday"] != 0),]

dates.str <- apply(as.matrix(gtdb.use[, c("iyear", "imonth", "iday")]), 1, paste, collapse="-")
library("lubridate")
adates <- ymd(dates.str)

countries <- unlist(c(gtdb.use[,"country_txt"]))
names(countries) <- NULL

##all.dates <- as.Date(ymd("1970-01-01"):ymd("2015-12-31"), origin="1970-01-01")
all.dates <- as.Date(ymd("1970-01-01"):ymd("2016-12-31"), origin="1970-01-01")
n.base <- rep(0, length(all.dates))
names(n.base) <- all.dates
y <- year(all.dates)
m <- month(all.dates)

gtdb.use.mult <- gtdb.use[gtdb.use[,"multiple"] == 1,]
gtdb.use.nmult <- gtdb.use[gtdb.use[,"multiple"] == 0,]



#### what fraction of the related attacks involve different cities?
library("maptools")
library("sp")
library("rgdal")
if (COUNTRY == "Afghanistan") {
    dname <- paste(proj.home, "/AFG_district_398/", sep="")
} else dname <- paste(proj.home, "/GADM/", sep="")


ogrListLayers(dsn=dname)

if(COUNTRY == "Pakistan") {
    shp <- readOGR(dsn=dname, layer="PAK_adm3")
} else if (COUNTRY == "India") {
    shp <- readOGR(dsn=dname, layer="IND_adm3")
} else if (COUNTRY == "Peru") {
    shp <- readOGR(dsn=dname, layer="PER_adm3")
} else if (COUNTRY == "United Kingdom") {
    shp <- readOGR(dsn=dname, layer="GBR_adm2")
    shp@data[,"NAME_3"] <- shp@data[,"NAME_2"]
} else if (COUNTRY == "El Salvador") {
    shp <- readOGR(dsn=dname, layer="SLV_adm2")
    shp@data[,"NAME_3"] <- shp@data[,"NAME_2"]
} else if (COUNTRY == "Philippines") {
    shp <- readOGR(dsn=dname, layer="PHL_adm2")
    shp@data[,"NAME_3"] <- shp@data[,"NAME_2"]
} else if (COUNTRY == "Colombia") {
    shp <- readOGR(dsn=dname, layer="COL_adm2")
    shp@data[,"NAME_3"] <- shp@data[,"NAME_2"]
} else if (COUNTRY == "Iraq") {
    shp <- readOGR(dsn=dname, layer="IRQ_adm2")
    shp@data[,"NAME_3"] <- shp@data[,"NAME_2"]
} else if (COUNTRY == "Afghanistan") {
    shp <- readOGR(dsn=dname, layer="district398")
    shp@data[,"NAME_3"] <- paste(shp@data[,"PROV_34_NA"], shp@data[,"DIST_34_NA"], sep=",")
}


## first, create a list of the related attacks
attacks.l <- list()
used <- rep(FALSE, nrow(gtdb.use.mult))
idx <- 1
repeat {
    i <- which(used == FALSE)[1]
    if(is.na(i))
        break

    rel.str <- strsplit(gtdb.use.mult$related[i], split=", ")[[1]]
    related <- gtdb.use.mult$eventid %in% rel.str
    related[i] <- TRUE
    this.attack <- gtdb.use.mult[related,]
    this.attack.df <- na.omit(data.frame(this.attack[,c("longitude", "latitude")]))

    if(nrow(this.attack.df) > 1) {
        pnts.tmp <- SpatialPoints(this.attack.df)
        proj4string(pnts.tmp) <- proj4string(shp)
    
        attacks.l[[idx]] <- cbind(over(pnts.tmp, shp), this.attack[rownames(this.attack.df),])
        idx <- idx + 1
    }
    used[related] <- TRUE
}

ndists <- sapply(attacks.l, function(x) length(unique(x[, "NAME_3"])))

table(ndists)

md.attacks.l <- attacks.l[ndists > 1]

dist.base <- rep(0, length(shp))
names(dist.base) <- shp@data[,"NAME_3"]

#### do we have many "claimed" attacks by Taliban in Afghanistan??
if(CLAIMED.ONLY) {
    all.claimed <- sapply(md.attacks.l, function(x) all(x[,"claimed"] != 0)) # about half are "claimed"
    md.attacks.use.l <- md.attacks.l[all.claimed]
} else if(TALIBAN.ONLY) {
    tal.attributed <- sapply(md.attacks.l, function(x) any(x[,"gname"] == "Taliban"))
    md.attacks.use.l <- md.attacks.l[tal.attributed]
} else md.attacks.use.l <- md.attacks.l


d.mat <- matrix(0, nrow=length(dist.base), ncol=length(md.attacks.use.l))
rownames(d.mat) <- names(dist.base)
for(idx in 1:length(md.attacks.use.l)) {
    dists <- unique(as.character(na.omit(md.attacks.use.l[[idx]][,"NAME_3"])))
    d.mat[dists, idx] <- 1
}

f <- function(x)
    apply(as.matrix(x[, c("iyear", "imonth", "iday")]), 1, paste, collapse="-")
dates.tmp <- lapply(md.attacks.use.l, f)
##sapply(dates.tmp, unique)  # basically all same day attacks
dates.str <- sapply(dates.tmp, function(x) x[1])
names(dates.str) <- NULL
library("lubridate")
adates <- ymd(dates.str)


#### igraph
shp.use <- shp@data[,"NAME_3"] %in% names(which(rowSums(d.mat) > 0))
library("spdep")
tmp <- poly2nb(shp[shp.use,], queen=FALSE)
adj.mat <- nb2mat(tmp, style="B", zero.policy=TRUE)
colnames(adj.mat) <- row.names(adj.mat) <- shp@data[shp.use,"NAME_3"]

library("igraph") #ONE-BASED INDEXING
b.graph <- graph.adjacency(adj.mat, mode="undirected", diag=FALSE)
xy.tmp <- coordinates(shp[shp.use,])
V(b.graph)$lon <- xy.tmp[, 1]
V(b.graph)$lat <- xy.tmp[, 2]

a.graph <- b.graph
#####

if(CLAIMED.ONLY) {
    ofname <- paste(proj.home, COUNTRY, "_GTDB_new_claimed.RData", sep="")
} else if (TALIBAN.ONLY) {
    ofname <- paste(proj.home, COUNTRY, "_GTDB_new_talattr.RData", sep="")
} else if (USE16) {
    ofname <- paste(proj.home, COUNTRY, "_GTDB_16.RData", sep="")
} else ofname <- paste(proj.home, COUNTRY, "_GTDB_new.RData", sep="")
save(d.mat, adates, a.graph, gtdb.use, file=ofname)
