proj.home <- "/home/usr0/z41440a/afghanistan_ecma_final/"
JOBID <- Sys.getenv("PJM_JOBID")

library("parallel")
print("cores:")
print(detectCores(logical=FALSE))
CORES <- as.numeric(Sys.getenv("PJM_VNODE_CORE"))
print(CORES)
if(is.na(CORES))
    CORES <- 3
options(mc.cores=CORES-1L)

options(echo=TRUE)

COUNTRY <- "Pakistan"  # "Afghanistan", "Pakistan", or "Monte" for simulations in Appendix G
CLAIMED.ONLY <- FALSE  # use only attacks claimed by the Taliban
TALATTR.ONLY <- FALSE  # use only attacks attributed to the Taliban
DROP.RAMADAN <- FALSE  # look at everything except Ramadan
RAMADAN.ONLY <- FALSE  # look at only Ramadan
RAMADAN.MONTHS <- FALSE  # create a separate month for Ramadan
stopifnot(DROP.RAMADAN + RAMADAN.ONLY + RAMADAN.MONTHS <= 1)

GEN.ALOC <- FALSE # for Figures 3, 7, 11, and N.10

GEN.BARX <- FALSE  # for Tables 2 and 5, and Figures 
GEN.BARX.SHUFFLE <- FALSE   # Tables 2 and 5 (columns I, III, IV) 
GEN.BARX.SHUFFLE.CROSSMONTH <- FALSE  # Tables 2 and 5 (column IV) - above has to be TRUE for this to have any effect
GEN.BARX.FLIP <- FALSE  # Tables 2 and 5 (columns II and V)
BOOT <- FALSE    # unused
BFRS.WITS <- TRUE  # use BFRS (for Pakistan) or WITS (for Afghanistan)
SIGACTS <- FALSE  ## use SIGACTS (only for Afghanistan)
SIGACTS.SMALL <- TRUE  ## cut off the low attack bits
stopifnot(BFRS.WITS + SIGACTS <= 1)  # default data is GTDB


EARLY.END <- FALSE   # for Figure 11, Table M.2
LATE.START <- FALSE  # for Figure N.11
GTDB.EARLY <- FALSE  # for Figure 10
GTDB.LATE <- FALSE   # for Figure 10
GTDB.FIGURE <- FALSE # produce Figure 10 or Figure N.11
stopifnot(GTDB.EARLY + GTDB.LATE <= 1)
if(GTDB.EARLY || GTDB.LATE)
    stopifnot(!BFRS.WITS && COUNTRY == "Afghanistan")
if(EARLY.END)
    stopifnot(BFRS.WITS && COUNTRY == "Pakistan")
if(LATE.START)
    stopifnot((!BFRS.WITS) && COUNTRY == "Pakistan")

if(COUNTRY != "Monte" && !BFRS.WITS) {
    if(GTDB.EARLY || GTDB.LATE || LATE.START) {
        print("ignoring args")
    } else {  # for Figure 9
        tmp.args <- commandArgs(trailingOnly=TRUE)
        print(paste("called with args", paste(tmp.args, collapse=",")))
        stopifnot(length(tmp.args) == 2)
        ## Af: 83 41 for 2014 and after
        ## Af: 14 13 for pre-Dorronsoro
        GTDB.KCENTRE <- as.numeric(tmp.args[1])
        GTDB.KWIDTH <- as.numeric(tmp.args[2])
        stopifnot(GTDB.KCENTRE > GTDB.KWIDTH)
    }
} else {
    tmp.args <- commandArgs(trailingOnly=TRUE)
    stopifnot(length(tmp.args) == 0)
}

GET.PVAL <- FALSE  # for Figure 9 and Tables 1, I.1, and M.2

TERRORISM.ONLY <- FALSE  # otherwise use all attacks
NON.NEGATIVE.COV <- TRUE  ## should generally be on
BYMONTH <- TRUE  ## should generally be TRUE for BFRS/WITS data, except for columns I and II of Tables 2 and 5 (must be false for GTDB data)
NO.KARACHI <- FALSE
LOW.KARACHI <- FALSE
SPLIT.KARACHI <- FALSE 
AF.GTDB.DROP <- FALSE # drop a couple of the largest attacks in afghanistan?

SKM.WEIGHTS <- TRUE # instead of covariance type weights (should generally be TRUE)
ADHOC.WEIGHTS <- TRUE # from before:  more attacks mean more information, so do a "partial" reweighting (should generally be TRUE)
MAX.WEIGHT <- 99  # leave unconstrained
MAX.ATTACKS <- 99  # leave unconstrained
TR.MIN <- TRUE  # perform trace minimization?  (should generally be on, but faster if off)
IGRAPH <- TRUE # should we do the test using geographic information? (should be on except for some appendix results)
FLIP.GEO <- TRUE  # should we alter geography rather than shuffling attacks?   (should be on except for some appendix results)

SK.GENETIC <- FALSE # do we use the complicated method of clustering?  (works slightly better than the simple one)
if(SK.GENETIC) {
    PCLUST <- TRUE  # seems to be better
    CLUTO <- FALSE  # (cluto and pclust give same results)
    stopifnot(PCLUST + CLUTO <= 1)
    MAXITER <- 12  # default is 12, but Krishna & Murty graphs go out to 200 iterations... (100+ seems worse, actually??)
}

if(COUNTRY == "Pakistan") {
    country.str <- "pak"
} else if (COUNTRY == "Afghanistan") {
    country.str <- "af"
} else country.str <- COUNTRY

fname.base <- paste("rep_", country.str, "_", ifelse(BYMONTH, "bymonth_", "nbm_"), sep="")


seed.tmp <- as.integer(Sys.time() + Sys.getpid())
print(seed.tmp)
set.seed(seed.tmp)
if(COUNTRY == "Monte") {
    library("Matrix")
    J <- 2
    N <- 50
    T <- 200

    Jmax <- N/2

    idxs <- round(seq(from=0, to=N, length.out=J+1))
    lens <- idxs[-1] - idxs[-length(idxs)]

    a <- lapply(lens, function(x) matrix(runif(x), nrow=x, ncol=1))  # need to make sure these stay the same mean...
    A <- bdiag(a)   # even slight differences here make things much more difficult?

    selected <- matrix(runif(J*T) < .1, ncol=J)
    lambda.mat <- apply(selected, 1, function(s) rowSums(A[,s,drop=FALSE]))/2
    d.l <- list()
    d.use.tmp <- matrix(rpois(prod(dim(lambda.mat)), lambda.mat), nrow=N)

    stopifnot(J %in% c(2,4))
    if(J == 4) {
        rownames(d.use.tmp) <- c(paste("a", 1:12, sep=""),
                                 paste("b", 1:13, sep=""),
                                 paste("c", 1:13, sep=""),
                                 paste("d", 1:12, sep=""))
    } else if (J == 2) {
        rownames(d.use.tmp) <- c(paste("a", 1:25, sep=""),
                                 paste("b", 1:25, sep=""))
    }
    colnames(d.use.tmp) <- paste("c", 1:ncol(d.use.tmp), sep="")
    d.use <- d.use.tmp[rowSums(d.use.tmp) > 0,]
    d.l[[1]] <- d.use
    mnths.use <- 1
    stopifnot(BYMONTH == FALSE)

} else if(COUNTRY == "Pakistan") {
    if(BFRS.WITS) {
        if(TERRORISM.ONLY) {
            ifname <- paste(proj.home, "pak_new_BFRS_terror.RData", sep="")
        } else if(SPLIT.KARACHI) {
            ifname <- paste(proj.home, "pak_new_BFRS_split.RData", sep="")
        } else ifname <- paste(proj.home, "pak_new_BFRS.RData", sep="")
        load(ifname)



        if(DROP.RAMADAN) {
            source(paste(proj.home, "ramadan.R", sep=""))
            not.ramadan <- !(ymd(colnames(d.mat)) %in% do.call("c", ramadan))
            d.m <- d.mat[,not.ramadan]
        } else if(RAMADAN.ONLY) {
            source(paste(proj.home, "ramadan.R", sep=""))
            is.ramadan <- (ymd(colnames(d.mat)) %in% do.call("c", ramadan))
            d.m <- d.mat[,is.ramadan]
        } else if(RAMADAN.MONTHS) {
            source(paste(proj.home, "ramadan.R", sep=""))
            is.ramadan <- (ymd(colnames(d.mat)) %in% do.call("c", ramadan))
            d.m <- d.mat

            ram.names <- gsub("(....)-(..-)", "\\1R00-", colnames(d.m))
            colnames(d.m)[is.ramadan] <- ram.names[is.ramadan]

            colnames(d.m)[colnames(d.m) == "2009-09-22"] <- "2009-10-a2"
            colnames(d.m)[colnames(d.m) == "2009-09-23"] <- "2009-10-a3"
            colnames(d.m)[colnames(d.m) == "2009-09-24"] <- "2009-10-a4"
            colnames(d.m)[colnames(d.m) == "2009-09-25"] <- "2009-10-a5"
            colnames(d.m)[colnames(d.m) == "2009-09-26"] <- "2009-10-a6"
            colnames(d.m)[colnames(d.m) == "2009-09-27"] <- "2009-10-a7"
            colnames(d.m)[colnames(d.m) == "2009-09-28"] <- "2009-10-a8"
            colnames(d.m)[colnames(d.m) == "2009-09-29"] <- "2009-10-a9"
            colnames(d.m)[colnames(d.m) == "2009-09-30"] <- "2009-10-a0"
        } else d.m <- d.mat

        mnths.all <- substr(colnames(d.m), start=1, stop=7)
        mnths <- unique(substr(colnames(d.m), start=1, stop=7))

        MSTART.str <- "2008-05"  ## standard start date
        if(EARLY.END) {
            MEND.str <- "2011-04"  # 6 months earlier
        } else MEND.str <- "2011-10"    ## standard end date
##        MSTART <- 245
##        MEND <- 286
        MSTART <- which(mnths == MSTART.str)
        MEND <- which(mnths == MEND.str)

        mnths.use <- mnths[MSTART:MEND]
        muse.all <- mnths.all[mnths.all %in% mnths.use]

        d.use.tmp <- pmin(d.m[, mnths.all %in% mnths.use], MAX.ATTACKS)
        d.use <- d.use.tmp[rowSums(d.use.tmp) > 0,]
        if(NO.KARACHI)
            d.use <- d.use[rownames(d.use) != "Karachi Central",]
        d.l <- list()
        if(BYMONTH) {
            for(mnth in mnths.use)
                d.l[[mnth]] <- d.use[, muse.all %in% mnth, drop=FALSE]
        } else {
            d.l[[1]] <- d.use
            mnths.use <- 1
        }
    } else {
        ifname <- paste(proj.home, "Pakistan_GTDB_new.RData", sep="")
        load(ifname)
        if(LATE.START) {
            d.mat11 <- d.mat[, -(1:17)]  # start in Nov. 2011
            d.use <- d.mat11[rowSums(d.mat11) > 0,]
        } else {
            obs.use <- (GTDB.KCENTRE - GTDB.KWIDTH):(GTDB.KCENTRE + GTDB.KWIDTH)
            d.mat.kernel <- d.mat[,obs.use]
            d.use <- d.mat.kernel[rowSums(d.mat.kernel) > 0,]
        }
        d.l <- list()
        d.l[[1]] <- d.use
        mnths.use <- 1
        stopifnot(BYMONTH == FALSE)
    }
} else if (COUNTRY == "Afghanistan") {
    library("rgdal")
    dname <- paste(proj.home, "/AFG_district_398/", sep="")
    ogrListLayers(dsn=dname)
    shp <- readOGR(dsn=dname, layer="district398", stringsAsFactors=FALSE)
    shp@data[,"NAME_3"] <- paste(shp@data[,"PROV_34_NA"], shp@data[,"DIST_34_NA"], sep=",")
    id.to.name <- shp@data[,"NAME_3"]
    names(id.to.name) <- as.character(shp@data[,"DISTID"])
    name.to.id <- shp@data[,"DISTID"]
    names(name.to.id) <- shp@data[,"NAME_3"]

    if(BFRS.WITS) {
        if(CLAIMED.ONLY) {
            ifname <- paste(proj.home, "af_newer_WITS.RData", sep="")
            load(ifname)
            d.m <- d.mat.tal[rowSums(d.mat.tal) > 0,]
        } else {
            ifname <- paste(proj.home, "af_newer_WITS.RData", sep="")
            load(ifname)

            if(RAMADAN.MONTHS) {
                source(paste(proj.home, "ramadan.R", sep=""))
                is.ramadan <- (ymd(colnames(d.mat)) %in% do.call("c", ramadan))
                d.m <- d.mat

                ram.names <- gsub("(....)-(..-)", "\\1R00-", colnames(d.m))
                colnames(d.m)[is.ramadan] <- ram.names[is.ramadan]

##            colnames(d.m)[colnames(d.m) == "2009-09-22"] <- "2009-10-a2"
##            colnames(d.m)[colnames(d.m) == "2009-09-23"] <- "2009-10-a3"
##            colnames(d.m)[colnames(d.m) == "2009-09-24"] <- "2009-10-a4"
##            colnames(d.m)[colnames(d.m) == "2009-09-25"] <- "2009-10-a5"
##            colnames(d.m)[colnames(d.m) == "2009-09-26"] <- "2009-10-a6"
##            colnames(d.m)[colnames(d.m) == "2009-09-27"] <- "2009-10-a7"
##            colnames(d.m)[colnames(d.m) == "2009-09-28"] <- "2009-10-a8"
##            colnames(d.m)[colnames(d.m) == "2009-09-29"] <- "2009-10-a9"
##            colnames(d.m)[colnames(d.m) == "2009-09-30"] <- "2009-10-a0"
            } else d.m <- d.mat
        }

        mnths.all <- substr(colnames(d.m), start=1, stop=7)
        mnths <- unique(substr(colnames(d.m), start=1, stop=7))

        mnths.use <- mnths
        muse.all <- mnths.all[mnths.all %in% mnths.use]

        d.use <- round(d.m)  # get rid of fractional attacks

        rownames(d.use) <- id.to.name[rownames(d.use)]
        stopifnot(!duplicated(rownames(d.use)))

        d.l <- list()
        if(BYMONTH) {
            for(mnth in mnths.use) {
                if(sum(muse.all %in% mnth) > 1) {
                    d.l[[mnth]] <- d.use[, muse.all %in% mnth]
                } else mnths.use <- setdiff(mnths.use, mnth)
            }
        } else {
            d.l[[1]] <- d.use
            mnths.use <- 1
        }
    } else if(SIGACTS) {
        ifname <- paste(proj.home, "af_new_WITS.RData", sep="")  # only for igraph graph
        load(ifname)
##        ifname <- paste(proj.home, "af_new_SIGACTS_DF.RData", sep="")
        ifname <- paste(proj.home, "af_new_SIGACTS.RData", sep="")
        load(ifname)

        d.m <- d.mat

        mnths.all <- substr(colnames(d.m), start=1, stop=7)
        mnths <- unique(substr(colnames(d.m), start=1, stop=7))
        mnths.use <- mnths

        muse.all <- mnths.all[mnths.all %in% mnths.use]

        if(SIGACTS.SMALL) {
            d.use <- d.m[rowSums(d.m) > rev(sort(rowSums(d.m)))[150],]
        } else d.use <- d.m

        rownames(d.use) <- id.to.name[rownames(d.use)]
        stopifnot(!duplicated(rownames(d.use)))

        d.l <- list()
        if(BYMONTH) {
            for(mnth in mnths.use)
                d.l[[mnth]] <- d.use[, muse.all %in% mnth]
        } else {
            d.l[[1]] <- d.use
            mnths.use <- 1
        }
    } else {
        if(CLAIMED.ONLY) {
            ifname <- paste(proj.home, "Afghanistan_GTDB_new_claimedtal.RData", sep="")
        } else if (TALATTR.ONLY) {
            ifname <- paste(proj.home, "Afghanistan_GTDB_new_talattr.RData", sep="")
        } else ifname <- paste(proj.home, "Afghanistan_GTDB_new.RData", sep="")
        load(ifname)
        ## total of 124 simultaneous attacks
        ## 80 gives one group, 85 splits into many...
        ## later attacks often lead to splits
        if(GTDB.EARLY) {
            obs.use <- 1:51    ## original version
        } else if (GTDB.LATE) {
            obs.use <- 74:124
        } else obs.use <- (GTDB.KCENTRE - GTDB.KWIDTH):(GTDB.KCENTRE + GTDB.KWIDTH)

        if(AF.GTDB.DROP) #take out the two largest attacks?
            obs.use <- setdiff(obs.use, c(40, 45))

        d.mat.kernel <- d.mat[,obs.use]
        d.use <- d.mat.kernel[rowSums(d.mat.kernel) > 0,]

        d.l <- list()
        d.l[[1]] <- d.use
        mnths.use <- 1
        stopifnot(BYMONTH == FALSE)
    }
} else {
    ifname <- paste(proj.home, country.str, "_GTDB_new.RData", sep="")
    load(ifname)

    obs.use <- (GTDB.KCENTRE - GTDB.KWIDTH):(GTDB.KCENTRE + GTDB.KWIDTH)

    d.mat.kernel <- d.mat[,obs.use]
    d.use <- d.mat.kernel[rowSums(d.mat.kernel) > 0,]

    d.l <- list()
    d.l[[1]] <- d.use
    mnths.use <- 1
    stopifnot(BYMONTH == FALSE)
}


if(IGRAPH) {
    library("igraph")
    if(COUNTRY == "Monte") {
        if(J == 4) {
            stopifnot(lens == c(12,13,13,12))
            tmp1 <- make_lattice(dimvector=c(3,4))
            V(tmp1)$name <- paste("a", 1:12, sep="")
            tmp2 <- induced_subgraph(make_lattice(dimvector=c(4,4)), 1:13)
            V(tmp2)$name <- paste("b", 1:13, sep="")
            tmp3 <- induced_subgraph(make_lattice(dimvector=c(4,4)), 1:13)
            V(tmp3)$name <- paste("c", 1:13, sep="")
            tmp4 <- make_lattice(dimvector=c(3,4))
            V(tmp4)$name <- paste("d", 1:12, sep="")
            g.use <- graph.union(tmp1, tmp2, tmp3, tmp4)
        } else if(J == 2) {
            stopifnot(lens == c(25, 25))
            tmp1 <- make_lattice(dimvector=c(5,5))
            V(tmp1)$name <- paste("a", 1:25, sep="")
            tmp2 <- make_lattice(dimvector=c(5,5))
            V(tmp2)$name <- paste("b", 1:25, sep="")
            g.use <- graph.union(tmp1, tmp2)
        }
    } else {
        if(COUNTRY == "Afghanistan") {
            ##        if(CLAIMED.ONLY) {
            if(BFRS.WITS) {  # graph is coded differently for some reason
                V(a.graph)$id <- V(a.graph)$name
                V(a.graph)$name <- id.to.name[V(a.graph)$name]
            }
            ##        } else V(a.graph)$id <- name.to.id[V(a.graph)$name] # this one now appears to be fixed
        }
        g.idxs <- which(V(a.graph)$name %in% rownames(d.use))   #### SIGACTS breaks here
        g.use <- induced_subgraph(a.graph, g.idxs)
        stopifnot(V(g.use)$name %in% rownames(d.use))
        stopifnot(rownames(d.use) %in% V(g.use)$name)
    }
}

if(BOOT) { #then we sample from d.l
### should we sample days rather than months?
##    d.l.tmp <- sample(d.l, replace=TRUE)
    boot.months <- sample(length(d.l), ncol(d.use), replace=TRUE)
    boot.obs <- lapply(boot.months, function(i) d.l[[i]][,sample(ncol(d.l[[i]]), 1), drop=FALSE])
    boot.tmp.l <- tapply(boot.obs, boot.months, FUN=c)
    d.l.tmp <- lapply(boot.tmp.l, do.call, what="cbind")
    for(i in 1:length(d.l.tmp))
        colnames(d.l.tmp[[i]]) <- NULL
    zeros <- rowSums(sapply(d.l.tmp, rowSums)) == 0
    d.l <- lapply(d.l.tmp, function(x) x[!zeros,,drop=FALSE])
}

library("lattice")
library("Matrix")
library("skmeans")
##library("igraph")


##### Greedy algorithm doesn't work - need to use an integer program instead
## get.flip <- function(d.orig) {
##     idxs.m.tmp <- which(d.orig > 0, arr.ind=TRUE)
##     if(nrow(idxs.m.tmp) <= 1)
##         return(d.orig)
##     mults.tmp <- d.orig[idxs.m.tmp]
##     ord <- order(mults.tmp, decreasing=TRUE)
##     idxs.m <- idxs.m.tmp[ord,]
##     mults <- mults.tmp[ord]
##     ## now everything should be correctly ordered:
##     stopifnot(d.orig[idxs.m] == mults)
##     csum.res <- colSums(d.orig)
##     d.shuffle <- matrix(0, nrow=nrow(d.orig), ncol=ncol(d.orig))
##     for(idx in 1:nrow(idxs.m)) {
##         i <- idxs.m[idx,"row"]
##         cols.avail <- which((csum.res >= mults[idx]) & (d.shuffle[i,] == 0))
##         if(length(cols.avail) == 0) {
##             warning("shuffle failed")
##             return(NULL)
##         } else if (length(cols.avail) == 1) {
##             j <- cols.avail
##         } else j <- sample(cols.avail, 1)
##         d.shuffle[i,j] <- mults[idx]
##         csum.res[j] <- csum.res[j] - mults[idx]
##         stopifnot(colSums(d.shuffle) + csum.res == colSums(d.orig))
##     }
##     colSums(d.shuffle) == colSums(d.orig)
##     return(d.shuffle)
## }

library("Rmosek")
### Constraints:  
### 	each day must have correct total number of attacks
###     each district must have correct *distribution* of attacks
###     no more than one "bunch" of attacks in any given district-day cell
###     (implement this last one as a "by cell" constraint?)
###
### How to set this up:
###     Variables are a certain "bunch" of attacks * district * day
###     We only need the variables for the district that actually has this "bunch"   
get.flip <- function(d.orig) {
    d <- d.orig[rowSums(d.orig) > 0, colSums(d.orig) > 0, drop=FALSE]
    if((nrow(d) <= 1) || (ncol(d) <= 1))  # no way of shuffling
        return(d.orig)
    vars.l <- list()
    bunches.l <- list()
    for(i in 1:nrow(d)) {
        bunches.l[[i]] <- sort(setdiff(unique(d[i,]),0))
        vnames <- paste(rownames(d)[i], outer(colnames(d), bunches.l[[i]], FUN=paste, sep="_"), sep="_")
        vars.l[[i]] <- matrix(vnames, nrow=ncol(d))
    }

    xlen <- length(unlist(vars.l))
    bunch.vals <- rep(unlist(bunches.l), each=ncol(d))
    names(bunch.vals) <- unlist(vars.l)
    milo1 <- list(sense = "max")

    milo1$c <- runif(xlen)

    A.colsums <- Matrix(0, nrow=ncol(d), ncol=xlen, sparse=TRUE)
    if(!is.null(colnames(d))) {
        rownames(A.colsums) <- paste("csum", colnames(d), sep="_")
    } else rownames(A.colsums) <- paste("csum", 1:ncol(d), sep="_")
    for(i in 1:ncol(d)) {

        thisday.vars <- ((1:xlen) %% ncol(d)) == (i %% ncol(d))
        A.colsums[i, thisday.vars] <- bunch.vals[thisday.vars]
    }

    blc.colsums <- colSums(d)
    buc.colsums <- colSums(d)

    ### A constraint here for each district * bunch (has to show up the right number of times)
    A.rowbunchsums <- Matrix(0, nrow=length(unlist(bunches.l)), ncol=xlen, sparse=TRUE)
    blc.rowbunchsums <- rep(NA, length(unlist(bunches.l)))
    for(i in 1:length(unlist(bunches.l))) {
        A.rowbunchsums[i, (1:ncol(d)) + (i-1)*ncol(d)] <- 1
        district <- rep(1:length(bunches.l), sapply(bunches.l, length))[i]
        blc.rowbunchsums[i] <- sum(d[district,] == unlist(bunches.l)[i])
        stopifnot(blc.rowbunchsums[i] > 0)
    }
    buc.rowbunchsums <- blc.rowbunchsums

    ### A constraint here that each district * day can only have at most one thing

    A.distday.l <- list()
    already.done.count <- c(0, cumsum(sapply(bunches.l, length)))
    for(dist in 1:nrow(d)) {
        if(length(bunches.l[[dist]]) > 1) {  ## then we actually need to do something
            A.distday.tmp <- Matrix(0, nrow=ncol(d), ncol=xlen, sparse=TRUE)
            idx.offset <- already.done.count[dist] * ncol(d)
            for(day in 1:ncol(d)) {
                idxs <- idx.offset + seq(day, by=ncol(d), length.out=length(bunches.l[[dist]]))
                A.distday.tmp[day, idxs] <- 1
            }
            A.distday.l[[paste("c", dist, sep=".")]] <- A.distday.tmp
        }
    }
    A.distday <- do.call("rbind", A.distday.l)
    if(!is.null(A.distday)) {
        blc.distday <- rep(0, nrow(A.distday))
        buc.distday <- rep(1, nrow(A.distday))
    } else {
        blc.distday <- NULL
        buc.distday <- NULL
    }

    A.covsums <- Matrix(0, nrow=nrow(d), ncol=xlen, sparse=TRUE)
    colsums.d <- colSums(d)
    for(dist in 1:nrow(d)) {
        bunches.l[[dist]]
        idx.offset <- already.done.count[dist] * ncol(d)
        idxs <- idx.offset + 1:(ncol(d) * length(bunches.l[[dist]]))
        coeffs <- colsums.d * rep(bunches.l[[dist]], each=ncol(d))
        A.covsums[dist, idxs] <- coeffs
    }
    blc.covsums <- colSums(t(d) * colsums.d)
    buc.covsums <- blc.covsums

    if(is.null(A.distday)) {
        milo1$A <- rbind(A.colsums, A.rowbunchsums, A.covsums)
    } else milo1$A <- rbind(A.colsums, A.rowbunchsums, A.distday, A.covsums)

    milo1$bc <- rbind(blc = c(blc.colsums, blc.rowbunchsums, blc.distday, blc.covsums),
                      buc = c(buc.colsums, buc.rowbunchsums, buc.distday, buc.covsums))

    milo1$bx <- rbind(blx = rep(0, xlen),
                      bux = rep(1, xlen))

    milo1$intsub <- 1:xlen

##    milo1$iparam <- list(PRESOLVE_USE=0, MIO_MAX_NUM_SOLUTIONS=1)
    milo1$iparam <- list(MIO_MAX_NUM_SOLUTIONS=1, NUM_THREADS=1, LOG_MIO_FREQ=1000)
    r <- mosek(milo1, opts=list(verbose=1))
##    r <- mosek(milo1, opts=list(verbose=4))
##    r <- mosek(milo1)
##    stopifnot(r$response$code == 0)
    xvec <- round(r$sol$int$xx)
    
    d.shuffle.tmp <- d * 0
    bunch.days <- rep(1:ncol(d), length(unlist(bunches.l)))
    bunch.dists <- rep(1:length(bunches.l), sapply(bunches.l, length) * ncol(d))
    xinds <- cbind(bunch.dists[xvec == 1], bunch.days[xvec == 1])
    vals <- bunch.vals[xvec == 1]
    d.shuffle.tmp[xinds] <- vals
    stopifnot(isTRUE(all.equal(colSums(d), colSums(d.shuffle.tmp))))
    stopifnot(isTRUE(all.equal(rowSums(d), rowSums(d.shuffle.tmp))))
    d.shuffle <- d.orig * 0
    d.shuffle[rowSums(d.orig) > 0, colSums(d.orig) > 0] <- d.shuffle.tmp
    return(d.shuffle)
}

get.shuffle <- function(d.orig) {
    if(ncol(d.orig) == 1)
        return(d.orig)
    d.shuffle <- matrix(nrow=nrow(d.orig), ncol=ncol(d.orig))
    dimnames(d.shuffle) <- dimnames(d.orig)
    for(i in 1:nrow(d.orig))
        d.shuffle[i,] <- sample(d.orig[i,], size=length(d.orig[i,]))
    return(d.shuffle)
}

get.shuffle.crossmonth <- function(d.orig.l) {
    ncols <- sapply(d.orig.l, ncol)
    sumcols <- c(0,cumsum(ncols))
    d.orig <- do.call("cbind", d.orig.l)
    d.shuffle <- matrix(nrow=nrow(d.orig), ncol=ncol(d.orig))
    dimnames(d.shuffle) <- dimnames(d.orig)
    for(i in 1:nrow(d.orig))
        d.shuffle[i,] <- sample(d.orig[i,], size=length(d.orig[i,]))
    d.shuffle.l <- list()
    for(i in 1:length(d.orig.l))
        d.shuffle.l[[i]] <- d.shuffle[,(sumcols[i]+1):sumcols[i+1],drop=FALSE]
    names(d.shuffle.l) <- names(d.orig.l)
    return(d.shuffle.l)
}

get.flip.old <- function(d.orig) {
    idxs.m <- which(d.orig > 0, arr.ind=TRUE)
    if(nrow(idxs.m) <= 1)
        return(d.orig)
    mults <- d.orig[d.orig > 0]
    attacks.l <- list()
    for(i in 1:length(mults)) {
        attacks.l[[i]] <- matrix(rep(idxs.m[i,], times=round(mults[i])), ncol=2, byrow=TRUE)
    }
    attacks.m <- do.call("rbind", attacks.l)
    attacks.shuffle <- sample(attacks.m[,2])
    d.shuffle <- sparseMatrix(attacks.m[,1], attacks.shuffle, x=1, dims=dim(d.orig), dimnames=dimnames(d.orig))
    stopifnot(round(colSums(d.shuffle)) == round(colSums(d.orig)))
    stopifnot(round(rowSums(d.shuffle)) == round(rowSums(d.orig)))
    return(d.shuffle)
}

##### trace minimization algorithm
trace.min <- function(Y) {
    N <- nrow(Y)
    mos <- list(sense="min")
##    mos$iparam$MSK_IPAR_NUM_THREADS <- CORES/2L

    mos$bardim <- c(N)

    mos$barA$i <- numeric(0)

    mos$barA$j <- rep(1, N*(N-1)/2 + N)
    mos$barA$k <- numeric(0)
    mos$barA$l <- numeric(0)
    mos$barA$v <- rep(1, N*(N-1)/2 + N)
    b <- numeric(0)

    ## constrain each entry of the off-diagonal separately...
    idx <- 1 
    for(l in 1:(N-1))
        for(k in (l+1):N) {
            mos$barA$i[idx] <- idx
            mos$barA$k[idx] <- k
            mos$barA$l[idx] <- l
            b[idx] <- 2*Y[k,l]  # the 2 here is both halves of the matrix?
            idx <- idx+1
        }
    for(d in 1:N) {
        mos$barA$i[idx] <- idx
        mos$barA$k[idx] <- d
        mos$barA$l[idx] <- d
        b[idx] <- Y[d,d]
        idx <- idx+1
    }

    len <- idx - 1

    ## minimize the diagonal
    mos$barc$j <- rep(1, N)
    mos$barc$k <- 1:N
    mos$barc$l <- 1:N

    mos$barc$v <- rep(1, nrow(Y))


    mos$A <- Matrix(0, ncol=1, nrow=len, sparse=TRUE)
    mos$c <- 0

    blc <- b
    blc[N*(N-1)/2 + (1:N)] <- 0  # diagonal has only upper bound
    mos$bc <- rBind(blc=blc, buc=b)

    mos$bx <- rBind(blx=0, bux=0)  # looks like it's needed to avoid a pointless error message...

    barx <- 1.0 * bandSparse(N, k=0:(1-N), symm=TRUE)

    mos$iparam <- list(NUM_THREADS=CORES)

    mos.out <- mosek(mos, opts=list(verbose=1))
    barx@x <- mos.out$sol$itr$barx[[1]]
    rownames(barx) <- rownames(Y)
    colnames(barx) <- colnames(Y)

    offdiag.err <- Y - barx
    diag(offdiag.err) <- 0
    summary(c(as.matrix(offdiag.err)))
    sort(diag(Y - barx))

    return(barx)
}

meths <- c("p", "i", "S", "s")

source(paste(proj.home, "cluster_base.R", sep=""))



get.L <- function(d.l) {
    if(!ADHOC.WEIGHTS) {
        Y.cov.vecm <- sapply(d.l, function(d) cov(as.matrix(t(d))))
        Y.cov.vec <- rowSums(Y.cov.vecm)
        Y.cov <- matrix(Y.cov.vec, nrow=nrow(d.l[[1]]), ncol=nrow(d.l[[1]]), dimnames=list(rownames(d.l[[1]]), rownames(d.l[[1]])))
        return(Y.cov)
    } else { # then ADHOC.WEIGHTS
        w.f <- function(x,y) sqrt(x*y)
        Y.l <- list()  

        for(i in 1:length(d.l)) {   # this is all for dropping situations where there are no attacks (often!)
            cov.m <- cov(as.matrix(t(d.l[[i]])))
            Y.l[[i]] <- t(t(cov.m / rowSums(d.l[[i]])) / rowSums(d.l[[i]]))
        }   # above weighting makes sense under assumption that some months just have "more bombs", and thus we need to adjust for that

        M.l <- list()
        for(i in 1:length(mnths.use)) {
            tmp <- rowSums(d.l[[i]])
            M.l[[i]] <- outer(tmp, tmp, w.f)
        }
        tmp.2 <- unlist(M.l)
        tmp.arr <- array(tmp.2, dim=c(dim(M.l[[1]]), length(M.l)))
        Y.2 <- unlist(Y.l)
        Y.arr <- array(Y.2, dim=c(dim(Y.l[[1]]), length(Y.l)))

        moments.tmp <- apply(tmp.arr * Y.arr, 1:2, sum, na.rm=TRUE)
        weights.sum <- apply(tmp.arr, 1:2, sum, na.rm=TRUE)
        moments.use <- moments.tmp / weights.sum
        moments.use[is.nan(moments.use)] <- 0

        stopifnot(dim(moments.use) == nrow(d.l[[1]]))
        rownames(moments.use) <- rownames(moments.tmp) <- rownames(d.l[[1]])
        colnames(moments.use) <- colnames(moments.tmp) <- rownames(d.l[[1]])
        return(moments.tmp)
    }

    Y <- pmax(Y.cov, 0)
    L <- -Y
    diag(L) <- diag(L) -rowSums(L)
    return(L)
}

if(GTDB.FIGURE) {
    Y.cov <- get.L(d.l)
    if(NON.NEGATIVE.COV)
        Y.cov <- pmax(Y.cov, 0)
    pdf("gtdb_figure.pdf")
    image(Y.cov, xaxt="n", yaxt="n")
    dev.off()
    pdf("gtdb_figure_bw.pdf")
    image(Y.cov, xaxt="n", yaxt="n", col=gray((0:11)/11, alpha=1))
    dev.off()

    q()
}

if(GEN.BARX) {
    if(!(GEN.BARX.FLIP||GEN.BARX.SHUFFLE)) {
        Y.cov <- get.L(d.l)
        if(NON.NEGATIVE.COV)
            Y.cov <- pmax(Y.cov, 0)
        barx <- trace.min(Y.cov)
        save(barx, Y.cov, file=paste(country.str, "_", ADHOC.WEIGHTS, "_", BYMONTH, "_barx.RData", sep=""))
        print(eigen(barx)$values)
    }
    if(GEN.BARX.FLIP) {
        for(i in 1:100) {
            d.la.flip <- mclapply(d.l, get.flip.old)
            Y.cov <- get.L(d.la.flip)
            if(NON.NEGATIVE.COV)
                Y.cov <- pmax(Y.cov, 0)
            barx <- trace.min(Y.cov)
            save(barx, Y.cov, file=paste(country.str, "_barx_flip/",
                                  country.str, "_", ADHOC.WEIGHTS, "_", BYMONTH, "_barx_flip_", JOBID, "_", i, ".RData", sep=""))
        }
    }
    if(GEN.BARX.SHUFFLE) {
        for(i in 1:100) {
            if(!GEN.BARX.SHUFFLE.CROSSMONTH) {
                d.la.shuffle <- mclapply(d.l, get.shuffle)
            } else d.la.shuffle <- get.shuffle.crossmonth(d.l)
            Y.cov <- get.L(d.la.shuffle)
            if(NON.NEGATIVE.COV)
                Y.cov <- pmax(Y.cov, 0)
            barx <- trace.min(Y.cov)
            if(BYMONTH) {
                ofname <- paste(country.str, "_barx_shuffle/", country.str, "_", ADHOC.WEIGHTS, "_barx_shuffle_", GEN.BARX.SHUFFLE.CROSSMONTH, "_", JOBID, "_", i, ".RData", sep="")
                save(barx, Y.cov, file=ofname)
            } else {
                ofname <- paste(country.str, "_barx_shuffle/", country.str, "_", ADHOC.WEIGHTS, "_", BYMONTH, "_barx_shuffle_", JOBID, "_", i, ".RData", sep="")
                save(barx, Y.cov, file=ofname)
            }
        }
    }
    q()
}





## Weighted graph partitioning from...
## Allow Me to Introduce
## Spectral and Isoperimetric Graph Partitioning
## Jonathan Richard Shewchuk (April 2016)

do.cut <- function(av, rewire=FALSE, groups=FALSE, cl=cl, NFLIPS=1000) {
    get.sweep.old <- function(L, groups=FALSE, minsize) {
        m <- diag(L)
        L.norm <- diag(1/sqrt(m)) %*% L %*% diag(1/sqrt(m))

        fvec.norm <- eigen(L.norm)$vectors[,nrow(L.norm)-1]

        fvec <- fvec.norm / sqrt(m)

        sweep.obj <- rep(NA, nrow(L)-1)

        ncut <- function(gr1, gr2) {
            cut <- sum(L[gr1, gr2]) + sum(L[gr2, gr1])
            if (cut == 0) {
                return(0)
            } else {
                vol1 <- sum(m[gr1])
                vol2 <- sum(m[gr2])
                adj <- (vol1 + vol2) / (vol1 * vol2)
                return(cut * adj)
            }
        }

        for(i in minsize:(nrow(L)-minsize)) {
            group1 <- order(fvec)[1:i]
            group2 <- order(fvec)[(i+1):nrow(L)]
            sweep.obj[i] <- ncut(group1, group2)
        }

        i <- which.max(sweep.obj)
        group1 <- sort(order(fvec)[1:i])
        group2 <- sort(order(fvec)[(i+1):nrow(L)])

        ## pseudo Kernighan-Lin (only swap one vertex at a time, because of weights)
        obj.cur <- ncut(group1, group2)
        repeat {
            improved <- FALSE
            for (j in 1:nrow(L)) {
                if(j %in% group1) {
                    g1a <- setdiff(group1, j)
                    g2a <- union(group2, j)
                } else {
                    g1a <- union(group1, j)
                    g2a <- setdiff(group2, j)
                }
                if(ncut(g1a, g2a) > obj.cur) {
                    group1 <- g1a
                    group2 <- g2a
                    improved <- TRUE
                    obj.cur <- ncut(group1, group2)
                }
            }
            if(!improved)
                break
        }

        ## end pseudo Kernighan-Lin

        if(groups) {
            return(list(groups=list(group1, group2), obj=obj.cur))
        } else return(obj.cur)
    }

    d.la <- lapply(d.l, function(d) d[av,,drop=FALSE])
    do.sk <- function(Y.cov, cl) {
        if(TR.MIN) {
            barx <- trace.min(Y.cov)
            if(SKM.WEIGHTS) {
                w <- pmin(sqrt(diag(barx)), MAX.WEIGHT)
            } else w <- rep(1, nrow(barx))
            Y.cor <- diag(sqrt(1/diag(barx))) %*% barx %*% diag(sqrt(1/diag(barx)))
        } else {
            if(SKM.WEIGHTS) {
                w <- pmin(sqrt(diag(Y.cov)), MAX.WEIGHT)
            } else w <- rep(1, nrow(Y.cov))
            Y.cor <- diag(sqrt(1/diag(Y.cov))) %*% Y.cov %*% diag(sqrt(1/diag(Y.cov)))
        }
        ei <- eigen(Y.cor)
        ei.use <- ei$values > 128 * .Machine$double.eps
        ei.vals <- ei$values[ei.use]
        ei.vecs <- ei$vectors[,ei.use]

        A.loc <- t(t(ei.vecs) * sqrt(ei.vals))
        rownames(A.loc) <- rownames(Y.cov)
        if(GEN.ALOC) {
            ofname <- paste(country.str, "cluster_save.RData", sep="_")
            ofname.global <- paste(country.str, "cluster_save_global.RData", sep="_")
            save(A.loc, w, file=ofname)
            save(COUNTRY, BFRS.WITS, SKM.WEIGHTS, TR.MIN, NON.NEGATIVE.COV, EARLY.END, meths, fname.base,
                 envir=.GlobalEnv, file=ofname.global)
            q()
        }
        if(!SK.GENETIC) {
            f <- function(x) {
                set.seed(round(runif(1)*10000) + x)
                skmeans(A.loc, k=2, weights = w, method="pclust", control = list(start=as.list(rep(meths, 100)), maxchains=10))
            }
            tmp.l <- clusterApplyLB(cl, 1:100, f)
            min.idx <- which.min(sapply(tmp.l, function(x) x$value))
            sk.out <- tmp.l[[min.idx]]
        } else {
            clusterExport(cl=cl, "A.loc", envir=environment())
            clusterExport(cl=cl, "w", envir=environment())
            sk.out <- my.sk(2, cl=cl, A.loc=A.loc, w=w)
        }
        return(sk.out)
    }

    do.norewire <- function(cl) {
        Y.cov <- get.L(d.la)
        if(NON.NEGATIVE.COV)
            Y.cov <- pmax(Y.cov, 0)
        sk.out <- do.sk(Y.cov, cl=cl)   # old afgh: 152.1582
        print("actual:")
        if(IGRAPH) {
            e1 <- ecount(induced_subgraph(g.use, which(V(g.use)$name %in% names(sk.out$cluster)[sk.out$cluster == 1])))
            print(e1)
            e2 <- ecount(induced_subgraph(g.use, which(V(g.use)$name %in% names(sk.out$cluster)[sk.out$cluster == 2])))
            print(e2)
            sk.out$value <- -(e1 + e2)
            print(sk.out$value)
            return(sk.out)
        } else {
            print(sk.out$value)
            return(sk.out)
        }
    }

    if(rewire == FALSE) {
        sk.out <- do.norewire(cl=cl)
    } else if (FLIP.GEO) {  # then we reshuffle geography rather than attacks
        stopifnot(IGRAPH)
        sk.out.actual <- do.norewire(cl=cl)  # start with the actual split
        sk.max.v <- NA
        for(i in 1:NFLIPS) {
            g1.rand <- sample(names(sk.out.actual$cluster), sum(sk.out.actual$cluster == 1))
            g2.rand <- names(sk.out.actual$cluster)[!(names(sk.out.actual$cluster) %in% g1.rand)]
            e1 <- ecount(induced_subgraph(g.use, which(V(g.use)$name %in% g1.rand)))
            e2 <- ecount(induced_subgraph(g.use, which(V(g.use)$name %in% g2.rand)))
            sk.max.v[i] <- -(e1 + e2)
        }
        return(sk.max.v)
    } else {
        sk.max.v <- NA
        sk.rand.l <- list()
        Y.cov.rand.l <- list()
        for(i in 1:100) {
            d.la.shuffle <- clusterApplyLB(cl, d.la, get.flip)
            Y.cov.rand <- get.L(d.la.shuffle)
            if(NON.NEGATIVE.COV)
                Y.cov.rand <- pmax(Y.cov.rand, 0)
            Y.cov.rand.l[[i]] <- Y.cov.rand
            sk.rand.l[[i]] <- do.sk(Y.cov.rand, cl=cl)
            if(IGRAPH) {
                e1 <- ecount(induced_subgraph(g.use, which(V(g.use)$name %in% names(sk.rand.l[[i]]$cluster)[sk.rand.l[[i]]$cluster == 1])))
                e2 <- ecount(induced_subgraph(g.use, which(V(g.use)$name %in% names(sk.rand.l[[i]]$cluster)[sk.rand.l[[i]]$cluster == 2])))
                sk.rand.l[[i]]$value <-  -(e1 + e2)
                sk.max.v[i] <- -(e1 + e2)
            } else sk.max.v[i] <- sk.rand.l[[i]]$value
            print("rand:")
            print(sk.rand.l[[i]]$value)
        }
        return(sk.max.v)
    }
}

avail <- rep(TRUE, nrow(d.l[[1]]))

cl <- makeForkCluster()

do.me <- function(a) {
    if(sum(a) < 2)
        return(list(a))
    tmp <- do.cut(a, rewire=FALSE, groups=TRUE, cl=cl)
##    rnd <- do.cut(a, rewire=TRUE, cl=cl, NFLIPS=100000)
     rnd <- do.cut(a, rewire=TRUE, cl=cl, NFLIPS=1000)
    if(GET.PVAL) {
        print(rnd)
        if(CLAIMED.ONLY) {
            if(BFRS.WITS) {
                ofname <- paste(proj.home, COUNTRY, "_pval_tal.RData", sep="")
            } else ofname <- paste(proj.home, COUNTRY, "_", GTDB.KCENTRE, "_", GTDB.KWIDTH, "_claimedtal.RData", sep="")
        } else if(TALATTR.ONLY) {
            if(BFRS.WITS) {
                ofname <- paste(proj.home, COUNTRY, "_pval_talattr.RData", sep="")
            } else ofname <- paste(proj.home, COUNTRY, "_", GTDB.KCENTRE, "_", GTDB.KWIDTH, "_talattr.RData", sep="")
        } else if(BFRS.WITS) {
            ofname <- paste(proj.home, COUNTRY, "_pval_pakf_", sum(a), "_", which.max(a), ".RData", sep="")
        } else ofname <- paste(proj.home, COUNTRY, "_", GTDB.KCENTRE, "_", GTDB.KWIDTH, ".RData", sep="")
        pval <- sum(rnd <= tmp$value)/length(rnd)
        save(tmp, rnd, pval, file=ofname)
        if(!BFRS.WITS) # then we only want the top level, and are done
            return(NULL)
    }
    if(sum(rnd <= tmp$value) >= .05 * length(rnd))
        return(list(a))
    ## otherwise cut
    g1 <- which(tmp$cluster == 1)
    g2 <- which(tmp$cluster == 2)
    if(sum(g2) > sum(g1)) {
        g.tmp <- g1
        g1 <- g2
        g2 <- g.tmp
    }
    print(tmp$value)
    print(g1)
    print(g2)
    
    a1 <- a
    a1[which(a1)[g2]] <- FALSE
    a2 <- a
    a2[which(a2)[g1]] <- FALSE
    return(list(do.me(a1), do.me(a2)))
}

out <- do.me(avail)
warnings()
print(length(out))
print(out)
