proj.home <- "/home/usr0/z41440a/afghanistan_ecma_final/"

library("parallel")
print("cores:")
print(detectCores(logical=FALSE))
CORES <- as.numeric(Sys.getenv("PJM_VNODE_CORE"))
print(CORES)
if(is.na(CORES))
    CORES <- 3
options(mc.cores=CORES-1L)
options(echo=TRUE)

tmp.args <- commandArgs(trailingOnly=TRUE)
print(paste("called with args", paste(tmp.args, collapse=",")))
stopifnot(length(tmp.args) == 2)
BY.YEAR <- as.logical(as.numeric(tmp.args[1]))
C.IDX <- as.numeric(tmp.args[2])

#####
##### Global Terrorism Database
#####

##library("readxl")
##origins <- data.frame(read_excel("Data_Origins of Terrorism.xlsx"))
##tmp <- read_excel("globalterrorismdb_0616dist.xlsx", guess_max=100000)
##gtdb <- tmp
##save(gtdb, file="globalterrorismdb_0616dist.RData")
ifname <- paste(proj.home, "globalterrorismdb_0617dist.RData", sep="")
load(ifname)

GTDB.SHORT <- FALSE
gtdb.use <- gtdb[(gtdb[,"imonth"] != 0) & (gtdb[,"iday"] != 0),]


dates.str <- apply(as.matrix(gtdb.use[, c("iyear", "imonth", "iday")]), 1, paste, collapse="-")
library("lubridate")
adates <- ymd(dates.str)

countries <- unlist(c(gtdb.use[,"country_txt"]))
names(countries) <- NULL

##all.dates <- as.Date(ymd("1970-01-01"):ymd("2015-12-31"), origin="1970-01-01")
if(GTDB.SHORT) {
    all.dates <- as.Date(ymd("2001-01-01"):ymd("2016-12-31"), origin="1970-01-01")
} else
    all.dates <- as.Date(ymd("1970-01-01"):ymd("2016-12-31"), origin="1970-01-01")
n.base <- rep(0, length(all.dates))
names(n.base) <- all.dates
y.u <- year(all.dates)
m <- month(all.dates)
mnth.u <- substr(all.dates, start=1, stop=7)


dis.l <- list()
countries.use <- unique(countries)
boot <- FALSE
country <- unique(countries)[C.IDX]

n.u <- n.base
attacks.tmp <- table(adates[countries == country])
attacks <- attacks.tmp[names(attacks.tmp) %in% names(n.u)]
n.u[names(attacks)] <- attacks

if(boot) {
    set.seed(runif(1)*100000 + boot)
    n <- sample(n.u, length(n.u), replace=TRUE)
    y <- as.numeric(substr(names(n), start=1, stop=4))
    mnth <- substr(names(n), start=1, stop=7)
} else {
    n <- n.u
    y <- y.u
    mnth <- mnth.u
}

attack.months <- sort(unique(substr(names(n), start=1, stop=7)))
year.substr <- substr(names(n), start=1, stop=4)

if(BY.YEAR) {
    attack.years <- sort(unique(year.substr))
} else attack.years <- "all"

ifname <- paste(proj.home, "dispersion_", BY.YEAR, "_", GTDB.SHORT, "/", "glm_", C.IDX, ".RData", sep="")
load(ifname)

##POIS.ITER <- 1000000  # out of memory
POIS.ITER <- 100000

ret <- rep(NA, length(attack.years))
names(ret) <- as.character(attack.years)

do.yr <- function(yr) {
    if(yr == "all") {
        yr.mnths <- attack.months
    } else yr.mnths <- attack.months[substr(attack.months, start=1, stop=4) == yr]

    if(length(reg.out$coef) > 1) {
        cfs <- reg.out$coef[paste("factor(mnth)", yr.mnths, sep="")]
    } else cfs <- reg.out$coef
    if(yr == "all") {
        max.obs <- max(n)
    } else max.obs <- max(n[y == yr])
    rpois.dist <- rep(0, max.obs+1)
    for(iter in 1:POIS.ITER) {
        rpois.l <- mapply(function(cf, num) rpois(num, exp(cf)), cfs, table(mnth)[yr.mnths])
        rpois.v <- unlist(rpois.l)
        if(max(rpois.v) > max.obs) {
            rpois.dist[(max.obs+2):(max(rpois.v)+1)] <- 0
            max.obs <- max(rpois.v)
        }
    for(i in 0:max.obs)
        rpois.dist[i+1] <- rpois.dist[i+1] + sum(rpois.v == i)/POIS.ITER
    }
    actual.dist <- rep(NA, max.obs+1)
    for(i in 0:max.obs)
        actual.dist[i+1] <- sum(n[mnth %in% yr.mnths] == i)

    names(rpois.dist) <- as.character(0:max.obs)

    csum.max <- max(cumsum(rev((actual.dist - rpois.dist) * as.numeric(names(rpois.dist)))))

    if(yr == "all") {
        ret.yr <- csum.max/sum(n)
    } else ret.yr <- csum.max/sum(n[y == yr])
    return(ret.yr)
}
ret <- mclapply(attack.years, do.yr)
names(ret) <- attack.years
dis.l[[country]] <- unlist(ret)

ofname <- paste(proj.home, "dispersion_", BY.YEAR, "_", GTDB.SHORT, "/", C.IDX, ".RData", sep="")
save(dis.l, file=ofname)
