proj.home <- "/home/usr0/z41440a/afghanistan_ecma_final/"

## library("readxl")
## origins <- data.frame(read_excel("Data_Origins of Terrorism.xlsx"))

## origins[origins[,"country"] == "Congo (Dem Rep.)", "country"] <- "Democratic Republic of the Congo"
## origins[origins[,"country"] == "Congo (Republic)", "country"] <- "Republic of the Congo"
## origins[origins[,"country"] == "Cote d'Ivoire", "country"] <- "Ivory Coast"
## origins[origins[,"country"] == "Mauretania", "country"] <- "Mauritania"
## origins[origins[,"country"] == "Trinidad", "country"] <- "Trinidad and Tobago"
## origins[origins[,"country"] == "UK", "country"] <- "United Kingdom"
## origins[origins[,"country"] == "USA", "country"] <- "United States"
## dat <- origins[!(origins[,"country"] %in% c("Mongolia", "Oman")) & origins[,"year"] != "1993",]
## save(dat, file="FKMS_dat.RData")

load("FKMS_dat.RData")

ifname <- paste(proj.home, "globalterrorismdb_0617dist.RData", sep="")
load(ifname)

gtdb.use <- gtdb[(gtdb[,"imonth"] != 0) & (gtdb[,"iday"] != 0),]
countries <- unlist(c(gtdb.use[,"country_txt"]))
names(countries) <- NULL

dates.str <- apply(as.matrix(gtdb.use[, c("iyear", "imonth", "iday")]), 1, paste, collapse="-")
library("lubridate")
adates <- ymd(dates.str)

all.dates <- as.Date(ymd("1970-01-01"):ymd("2016-12-31"), origin="1970-01-01")
n.base <- rep(0, length(all.dates))
names(n.base) <- all.dates

fragmentation.l <- list()
fragmentation.max.l <- list()
for(country in unique(countries)) {
    attacks <- table(adates[countries == country])
    n <- n.base
    n[names(attacks)] <- attacks
    gtdb.country <- gtdb.use[countries == country,]
    attack.yrs <- unclass(gtdb.country[, "iyear"])[[1]]
    frag.tmp.l <- list()
    frag.weight.l <- list()
    frag.max.l <- list()
    for (yr in unique(attack.yrs)) {
      grps.tmp <- table(gtdb.country[gtdb.country[,"iyear"] == yr, "gname"])
      grps <- grps.tmp[names(grps.tmp) != "Unknown"]
      frag.tmp.l[[as.character(yr)]] <- 1 - sum((grps/sum(grps))^2)
      frag.weight.l[[as.character(yr)]] <- min(max(sum(grps) - 1, 0),1)
      grps.max <- rep(1, sum(grps))
      frag.max.l[[as.character(yr)]] <- 1 - sum((grps.max/sum(grps))^2)
    }
    frag.mat <- cbind(unlist(frag.tmp.l), unlist(frag.weight.l))
    frag.max.mat <- cbind(unlist(frag.max.l), unlist(frag.weight.l))
    fragmentation.l[[country]] <- frag.mat[frag.mat[,2] == 1, 1]
    fragmentation.max.l[[country]] <- frag.max.mat[frag.max.mat[,2] == 1, 1]
}

dat[,"frag"] <- NA
dat[,"frag.max"] <- NA
for(country in names(fragmentation.l)) {
    for(yr in names(fragmentation.l[[country]])) {
        dat[dat[,"country"] == country & dat[,"year"] == yr, "frag"] <- fragmentation.l[[country]][yr]
        dat[dat[,"country"] == country & dat[,"year"] == yr, "frag.max"] <- fragmentation.max.l[[country]][yr]
    }
}

dname <- paste(proj.home, "dispersion_TRUE_FALSE/", sep="")
dispersion.l <- list()
for(c.idx in 1:205) {
    ifname <- paste(dname, c.idx, "d.RData", sep="")
    load(ifname)
    dispersion.l[[names(dis.l)]] <- dis.l[[1]]
}

stopifnot(unique(dat[,"country"]) %in% names(dispersion.l))

dat[,"dispersion"] <- NA
for(i in 1:nrow(dat)) {
    tmp <- dispersion.l[[dat[i,"country"]]][as.character(dat[i, "year"])]
    if(is.finite(tmp))
        dat[i,"dispersion"] <- tmp
}



library("AER")
library("apsrtable")

reg1 <- lm(frag ~ dispersion, data=dat)
reg1$se <- vcovHC(reg1)
reg2 <- lm(frag ~ dispersion + frag.max + gdp + I(gdp^2) + consum + trade + investment + populationsize + govsize + democracy + stability + civilwar, data=dat)
reg2$se <- vcovHC(reg2)
reg3 <- lm(frag ~ dispersion + frag.max + gdp + I(gdp^2) + consum + trade + investment + populationsize + govsize + democracy + stability + civilwar + factor(country) - 1, data=dat)
reg3$se <- vcovHC(reg3, type="HC0")
reg4 <- lm(frag ~ dispersion + frag.max + gdp + I(gdp^2) + consum + trade + investment + populationsize + govsize + democracy + stability + civilwar + factor(year) + factor(country) - 1, data=dat)
reg4$se <- vcovHC(reg4, type="HC0")
tab.out <- apsrtable(reg1,reg2,reg3,reg4,model.names=c("I", "II", "III", "IV"), label="tab:frag", omitcoef=expression(grep("factor",coefnames)))
ofname <- paste(proj.home, "tab_frag.tex", sep="")
writeLines(tab.out, con=ofname)


