proj.home <- "/home/usr0/z41440a/afghanistan_ecma_final/"

library("AER")
library("ivpack")

## library("readxl")
## origins <- data.frame(read_excel("Data_Origins of Terrorism.xlsx"))

## origins[origins[,"country"] == "Congo (Dem Rep.)", "country"] <- "Democratic Republic of the Congo"
## origins[origins[,"country"] == "Congo (Republic)", "country"] <- "Republic of the Congo"
## origins[origins[,"country"] == "Cote d'Ivoire", "country"] <- "Ivory Coast"
## origins[origins[,"country"] == "Mauretania", "country"] <- "Mauritania"
## origins[origins[,"country"] == "Trinidad", "country"] <- "Trinidad and Tobago"
## origins[origins[,"country"] == "UK", "country"] <- "United Kingdom"
## origins[origins[,"country"] == "USA", "country"] <- "United States"
## dat <- origins[!(origins[,"country"] %in% c("Mongolia", "Oman")) & origins[,"year"] != "1993",]
## save(dat, file="FKMS_dat.RData")

load("FKMS_dat.RData")

dat[,"logattacks"] <- log(dat[,"terrorattacks"])
dat[!is.finite(dat[,"logattacks"]), "logattacks"] <- NA
dat[,"gdpsq"] <- dat[,"gdp"]^2 / 1000000

dname <- paste(proj.home, "dispersion_TRUE_FALSE/", sep="")
dispersion.l <- list()
for(c.idx in 1:205) {
    ifname <- paste(dname, c.idx, "d.RData", sep="")
    load(ifname)
    dispersion.l[[names(dis.l)]] <- dis.l[[1]]
}

stopifnot(unique(dat[,"country"]) %in% names(dispersion.l))

dat[,"dispersion"] <- NA
dat[,"dispersion.lag"] <- NA
for(i in 1:nrow(dat)) {
    tmp <- dispersion.l[[dat[i,"country"]]][as.character(dat[i, "year"])]
    if(is.finite(tmp))
        dat[i,"dispersion"] <- tmp

    tmp <- dispersion.l[[dat[i,"country"]]][as.character(dat[i, "year"]-1)]
    if(is.finite(tmp))
    dat[i,"dispersion.lag"] <- tmp
}



reg1 <- lm(logattacks ~ dispersion, data=dat)
reg1$se <- sqrt(diag(vcovHC(reg1)))
reg2 <- lm(logattacks ~ dispersion + gdp + gdpsq + consum + trade + investment + populationsize + govsize + democracy + stability + civilwar + factor(year) + factor(country) - 1, data=dat)
reg2$se <- sqrt(diag(vcovHC(reg2, type="HC0")))


ireg1 <- ivreg(logattacks ~ dispersion | dispersion.lag, data=dat)
ireg1$se <- robust.se(ireg1)[,"Std. Error"]
ireg2 <- ivreg(logattacks ~ dispersion + gdp + gdpsq + consum + trade + investment + populationsize + govsize + democracy + stability + civilwar + factor(year) + factor(country) - 1 | dispersion.lag + gdp + gdpsq + consum + trade + investment + populationsize + govsize + democracy + stability + civilwar + factor(year) + factor(country) - 1, data=dat)
ireg2$se <- robust.se(ireg2)[,"Std. Error"]

library("stargazer")
ofname <- paste(proj.home, "tab_logattacks.tex", sep="")
stargazer(reg1, reg2, ireg1, ireg2, se=list(reg1$se, reg2$se, ireg1$se, ireg2$se), omit=c("factor"), float=FALSE, out=ofname)

tab.out <- apsrtable(reg1,reg2,ireg1,ireg2,model.names=c("I", "II", "III", "IV"), label="tab:logattacks_iv", omitcoef=expression(grep("factor",coefnames)))
ofname <- paste(proj.home, "tab_logattacks.tex", sep="")
writeLines(tab.out, con=ofname)

##### Table N8
ifname <- paste(proj.home, "globalterrorismdb_0617dist.RData", sep="")
load(ifname)

gtdb.use <- gtdb[(gtdb[,"imonth"] != 0) & (gtdb[,"iday"] != 0),]
countries <- unlist(c(gtdb.use[,"country_txt"]))
names(countries) <- NULL

dates.str <- apply(as.matrix(gtdb.use[, c("iyear", "imonth", "iday")]), 1, paste, collapse="-")
library("lubridate")
adates <- ymd(dates.str)

related <- !is.na(gtdb.use[,"related", drop=FALSE])[,1]   # 13% are related
related.ag <- by(related, list(countries, substr(adates, 1, 4)), FUN=mean)
dat[,"related"] <- related.ag[as.matrix(dat[,c("country","year")])]

reg1 <- lm(related ~ dispersion, data=dat)
reg1$se <- vcovHC(reg1)
reg2 <- lm(related ~ dispersion + gdp + I(gdp^2) + consum + trade + investment + populationsize + govsize + democracy + stability + civilwar, data=dat)
reg2$se <- vcovHC(reg2)
reg3 <- lm(related ~ dispersion + gdp + I(gdp^2) + consum + trade + investment + populationsize + govsize + democracy + stability + civilwar + factor(country) - 1, data=dat)
reg3$se <- vcovHC(reg3, type="HC0")
reg4 <- lm(related ~ dispersion + gdp + I(gdp^2) + consum + trade + investment + populationsize + govsize + democracy + stability + civilwar + factor(year) + factor(country) - 1, data=dat)
reg4$se <- vcovHC(reg4, type="HC0")

library("apsrtable")

tab.out <- apsrtable(reg1,reg2,reg3,reg4,model.names=c("I", "II", "III", "IV"), label="tab:related", omitcoef=expression(grep("factor",coefnames)))
ofname <- paste(proj.home, "tab_related.tex", sep="")
writeLines(tab.out, con=ofname)

