COUNTRY <- "Afghanistan"
BYMONTH <- TRUE
FLIP <- FALSE
SHUFFLE <- TRUE
CROSSMONTH <- TRUE
ADHOC.WEIGHTS <- FALSE

FIGURE4 <- FALSE

if(COUNTRY == "Pakistan") {
    country.str <- "pak"
} else if (COUNTRY == "Afghanistan") {
    country.str <- "af"
} else country.str <- COUNTRY

if(SHUFFLE) {
    idname <- paste(country.str, "_barx_shuffle", sep="")
    if(BYMONTH) {
        regex <- paste(country.str, "_", ADHOC.WEIGHTS, "_barx_shuffle_", CROSSMONTH, "_[0-9]*_[0-9]*.RData", sep="")
    } else regex <- paste(country.str, "_", ADHOC.WEIGHTS, "_", BYMONTH, "_barx_shuffle_[0-9]*_[0-9]*.RData", sep="")
    ifnames <- list.files(path=idname, pattern=regex, full.names=TRUE)
} else if (FLIP) {
    idname <- paste(country.str, "_barx_flip", sep="")
    ifnames <- list.files(path=idname, pattern=paste(country.str, "_", ADHOC.WEIGHTS, "_", BYMONTH, "_barx_flip_[0-9]*_[0-9]*.RData", sep=""), full.names=TRUE)
} else ifnames <- paste(country.str, "_", ADHOC.WEIGHTS, "_", BYMONTH, "_barx.RData", sep="")

library("parallel")
library("Matrix")

NGROUPS <- 2  # 2 for Afghanistan, 4 for Pakistan
K.SCREE <- 20   # go up higher than for the csv output
for(ifname in ifnames) {
    load(ifname)
##    print(ifname)
    S <- barx
    ei <- eigen(S)
    if(FIGURE4) {
        library("lattice")
        er <- ei$values[-length(ei$values)] / ei$values[-1]
        pdf("af_eigenratio_colour.pdf")
        xyplot(er[1:100] ~ 1:100, ylab="Eigenvalue Ratio", xlab="Index")
        dev.off()
        pdf("af_eigenratio_bw.pdf")
        xyplot(er[1:100] ~ 1:100, ylab="Eigenvalue Ratio", xlab="Index", col="black")
        dev.off()
        q()
    }

##    print(ei$values)
##    if(all(ei$vectors[,1] > -sqrt(.Machine$double.eps))) {
##        B <- t(t(ei$vectors) * sqrt(ei$values))
##    } else B <- -t(t(ei$vectors) * sqrt(ei$values)) # negative helps with K=1
    stopifnot(min(ei$values) > -sqrt(.Machine$double.eps))
    if(all(ei$vectors[,1] > -sqrt(.Machine$double.eps))) {
        B <- t(t(ei$vectors) * sqrt(pmax(ei$values,0)))
    } else B <- -t(t(ei$vectors) * sqrt(pmax(ei$values,0))) # negative helps with K=1
    hss <- function(K) {
        ##        B <- -t(t(ei$vectors[,1:K]) * sqrt(ei$values[1:K]))   # negative helps with K=1
        ##        sum(S - B %*% t(B))
        ##        B <- -B.full   

        ##        Q <- diag(nrow=ncol(B))
        Q <- rbind(diag(nrow=K), matrix(0, nrow=nrow(S)-K, ncol=K))

        iter <- 0
        tr <- Inf
        while(abs(tr) > 1e-12) {
            W <- apply(B %*% Q, 1:2, max, 0)
            Q <- with(svd(t(W) %*% B), v %*% t(u))
            tr <- sum(diag(t(W) %*% (W - B %*% Q)))
            ##            print(tr)
            iter <- iter + 1
            if(iter > 1000000)
                stop("HSS: iteration limit")
        }
        print(sum((S - W %*% t(W))^2))
        colnames(W) <- paste("W", K, "g", 1:K, sep="")

        return(W)
    }

    ## doesn't appear to work for K=1 (drop=FALSE?)
    W.l <- lapply(1:K.SCREE, hss)
    W.share.l <- lapply(W.l, function(W) W/rowSums(W))

    hss.rss <- function(W)  # Frobenius norm on residual
        sqrt(sum((S - W %*% t(W))^2))
    ##        sum((S - W %*% t(W))^2)
    ##        sum((W %*% t(W))^2)/sum(S^2)
    ##        sum(W^2)/sum(B.full^2)
    rss <- c(sqrt(sum(S^2)), sapply(W.l, hss.rss))  # but we want the additional contribution of each new group...
    gap <- rss[-length(rss)] - rss[-1]

    ## xyplot(rss ~ 0:K.SCREE)    
    ## xyplot(gap ~ 1:K.SCREE)    
    fname.base <- substr(ifname,start=1, stop=nchar(ifname)-6)

    if(!(SHUFFLE||FLIP)) {
        W.df <- data.frame(do.call("cbind", W.l[1:NGROUPS]))
        W.share.df <- data.frame(do.call("cbind", W.share.l[1:NGROUPS]))
        rownames(W.df) <- rownames(barx)
        rownames(W.share.df) <- rownames(barx)
        hssfname <- paste(fname.base, "_hss.csv", sep="")
        hsssfname <- paste(fname.base, "_hss_share.csv", sep="")
        write.csv(W.df, file=hssfname)
        write.csv(W.share.df, file=hsssfname)
        csvt <- c('"String"', rep('"Real"', ncol(W.df)))
        writeLines(csvt, con=paste(hssfname,"t",sep=""), sep=",")
        writeLines(csvt, con=paste(hsssfname,"t",sep=""), sep=",")
    }
    tmp <- c(sum(S^2), sapply(W.l, function(W) sum((S - W %*% t(W))^2)))
    tmp2 <- tmp[-length(tmp)] - tmp[-1]
    tmp3 <- tmp2[-length(tmp2)]/tmp2[-1]
    hssfname <- paste(fname.base, "_hss.RData", sep="")
    save(S, W.l, tmp, tmp2, tmp3, file=hssfname)
}

q()

if(SHUFFLE) {
    regex <- paste(country.str, "_", ADHOC.WEIGHTS, "_barx_shuffle_", CROSSMONTH, "_[0-9]*_[0-9]*_hss.RData", sep="")
    ifnames <- list.files(path=idname, pattern=regex, full.names=TRUE)
} else ifnames <- list.files(path=idname, pattern=paste(idname, "_[0-9]*_[0-9]*_hss.RData", sep=""), full.names=TRUE)
out.l <- list()
for(ifname in ifnames) {
    load(ifname)
##    out.l[[ifname]] <- tmp3[1]
    out.l[[ifname]] <- max(tmp3)
}
sort(unlist(out.l))
